;==========================================================================
;
; Copyright 2010, Phil Lewis (phlew8 at aol.com)
;
; Many thanks to:
;   * Joe Zbiciak for the jzIntv emulator, the as1600 assembler, tons of
;     documentation and example code, optimizations, ideas, and general 
;     encouragement 
;
;   * Arnauld Chevallier for the "Tracker" music tracker, which saved me
;     many hours putting together the music track
;
;   * Intelliwiki for the handy online documentation and tutorials
;     (http://intelliwiki.kylesblog.com/index.php/Main_Page)
;
;==========================================================================

                    ; 16-bit ROM width
                    ROMW        16

;------------------------------------------------------------------------------
; Constants
;------------------------------------------------------------------------------
                    INCLUDE     "gimini.asm"

ISR_STATES          EQU         5
MOB_UPD_TICKS       EQU         3   ; Update the MOB every X ticks
H_SCR_UPD_TICKS     EQU         4   ; Update the h_delay value every X ticks
MOB_X_VEL_INIT      EQU         2   ; MOB's initial X velocity
MOB_Y_VEL_INIT      EQU         1   ; MOB's initial Y velocity

;------------------------------------------------------------------------------
; Scratch-pad and System RAM allocations 
;------------------------------------------------------------------------------
                    INCLUDE     "ram-alloc.asm"
                    
;==============================================================================
; Program code starts
;==============================================================================

;------------------------------------------------------------------------------
; EXEC-friendly ROM header.
;
            ORG     $5000           ; Use default memory map
ROMHDR      BIDECLE ZERO            ; MOB picture base   (points to NULL list)
            BIDECLE ZERO            ; Process table      (points to NULL list)
            BIDECLE MAIN            ; Program start address
            BIDECLE ZERO            ; Bkgnd picture base (points to NULL list)
            BIDECLE $0101           ; GRAM pictures
            BIDECLE TITLE           ; Cartridge title/date
            DECLE   $03C0           ; No ECS title, run code after title,
                                    ; ... no clicks
ZERO        DECLE   $0000           ; Screen border control
            DECLE   $0000           ; 0 = color stack, 1 = f/b mode
            DECLE   0, 0, 0, 0, 0   ; Initial color stack and border:  Black

;------------------------------------------------------------------------------
;  Title screen
;
TITLE       PROC
            BYTE    110, "Dancin' Baby", 0
            BEGIN

            ; Patch the title string to ours instead of Mattel.
            CALL    PRINT.FLS
            DECLE   C_WHT
            DECLE   MEMMAP.backtab + (20 * 3) + 1
            STRING  '    '
            STRING  'Phil Lewis'
            STRING  '    '
            BYTE    0

            CALL    PRINT.FLS
            DECLE   C_WHT
            DECLE   MEMMAP.backtab + $CF    ; Point to 'Mattel' in lower-right
            STRING  '2010 PGLewis'
            BYTE    0

            ; Done.
            RETURN                  ; Return to EXEC for title screen display
            ENDP

;------------------------------------------------------------------------------
;  MAIN:  Entry point after the title screen
;
MAIN        PROC

            ; Move the stack pointer, prep, and enable interrupts
            MVII    #STACK, R6
            CALL    INITIALIZE
            EIS

            ; All the work is done in the ISR
@@forever   B       @@forever

            ENDP

;------------------------------------------------------------------------------
;  Preparation work
;
INITIALIZE  PROC
            BEGIN

            ; Setup the tracker
            CALL    TRKINIT         ; initialize tracker
            CALL    TRKSNGINIT      ; initialize song
            DECLE   SONG00
            
            ; Initialize the mode sequence     
            MVII    #MODES, R0
            MVO     R0, THIS_MODE
            MVII    #2, R0
            MVO     R0, MODE_COUNTER
            
            ; Setup the ISR
            MVII    #ISR, R0
            MVO     R0, ISRVEC1
            SWAP    R0
            MVO     R0, ISRVEC2

            RETURN
            ENDP
;------------------------------------------------------------------------------
;  Load the backtab
;
LOAD_BTAB   PROC
            
            MVI@    R5, R4              ; R4 -> backtab data
            PSHR    R5                  ; Save return address
            MVII    #MEMMAP.backtab, R5 ; R5 -> start of the backtab
            MVII    #240, R1            ; R1 = counter to move all cards (20x12)

@@loop
            MVI@    R4, R0              ; Get a value from the specified data table
            MVO@    R0, R5              ; Write to backtab
            DECR    R1                  ; Decrement the counter
            BNEQ    @@loop              ; and loop until we're done

            PULR    PC
            ENDP

;------------------------------------------------------------------------------ 
;  The bulk of the work is done here in the interrupt handler

ISR         PROC

            CALL    UPD_STIC
            CALL    RUN_MODE
            CALL    TRKPLAY

            B       $1014
            ENDP

;------------------------------------------------------------------------------
;
UPD_STIC    PROC
            BEGIN

            ; Refresh video registers
            MVO     R0, STIC.viden
            MVI     STIC.mode, R0
            MVI     BORD_COLOR, R0
            MVO     R0, STIC.bord
            MVI     CS0_COLOR, R0
            MVO     R0, STIC.cs0
            MVI     CS1_COLOR, R0
            MVO     R0, STIC.cs1
            MVI     CS2_COLOR, R0
            MVO     R0, STIC.cs2
            MVI     CS3_COLOR, R0
            MVO     R0, STIC.cs3
            
            RETURN
            ENDP

;------------------------------------------------------------------------------
;
RUN_MODE    PROC
            BEGIN

            MVI     THIS_MODE, R4
            MVI     MODE_COUNTER, R0
            DECR    R0
            BEQ     @@next_mode
            MVI@    R4, R5
            B       @@do_mode
            
@@next_mode            
            ADDI    #2, R4
            CMPI    #MODES.end, R4
            BNEQ    @@save_mode
            MVII    #MODES, R4
    
@@save_mode                 
            MVO     R4, THIS_MODE
            MVI@    R4, R5          ; Mode's procedure address
            MVI@    R4, R0          ; Number of frames to run this new mode
            
@@do_mode                        
            MVO     R0, MODE_COUNTER
            MVII    #@@ret, R0
            PSHR    R0
            JR      R5
            
@@ret            
            RETURN
            ENDP  
                      
;------------------------------------------------------------------------------
; Caller needs to prep: 
;
; R4 -> Cards' source
; R1 -> GRAM starting loc (destination)
; R0 = Number of cards
;
COPY_CARDS  PROC
            BEGIN

            MOVR    R1, R5      ; Get the destination in R5

@@copy
            MVI@    R4, R1
            MVO@    R1, R5
            SWAP    R1
            MVO@    R1, R5

            MVI@    R4, R1
            MVO@    R1, R5
            SWAP    R1
            MVO@    R1, R5

            MVI@    R4, R1
            MVO@    R1, R5
            SWAP    R1
            MVO@    R1, R5

            MVI@    R4, R1
            MVO@    R1, R5
            SWAP    R1
            MVO@    R1, R5
            DECR    R0
            BNZE    @@copy

            RETURN
            ENDP

;========================================================================
;  INCLUDES
;========================================================================
            INCLUDE     "mode-lg.asm"
            INCLUDE     "mode-sm.asm"
            INCLUDE     "mode-tiny.asm"
            INCLUDE     "print.asm"
            INCLUDE     "tracker.mac"
            INCLUDE     "tracker.asm"
            INCLUDE     "hooked.asm"
            INCLUDE     "data.asm"

;* ======================================================================== *;
;*  This program is free software; you can redistribute it and/or modify    *;
;*  it under the terms of the GNU General Public License as published by    *;
;*  the Free Software Foundation; either version 2 of the License, or       *;
;*  (at your option) any later version.                                     *;
;*                                                                          *;
;*  This program is distributed in the hope that it will be useful,         *;
;*  but WITHOUT ANY WARRANTY; without even the implied warranty of          *;
;*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU       *;
;*  General Public License for more details.                                *;
;*                                                                          *;
;*  You should have received a copy of the GNU General Public License       *;
;*  along with this program; if not, write to the Free Software             *;
;*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               *;
;* ======================================================================== *;
;*                   Copyright (c) 2010, Phil Lewis                         *;
;* ======================================================================== *;
